@@ -23,21 +23,21 @@ failing examples it finds.
 """
 
 
-from hypothesis.searchstrategy.strategies import strategy
-from hypothesis._settings import settings, Verbosity, Settings
+from hypothesis._settings import settings, Verbosity, Phase, HealthCheck
 from hypothesis.version import __version_info__, __version__
-from hypothesis.control import assume, note
+from hypothesis.control import assume, note, reject
 from hypothesis.core import given, find, example, seed
 
 
 __all__ = [
     'settings',
-    'Settings',
     'Verbosity',
+    'HealthCheck',
+    'Phase',
     'assume',
+    'reject',
     'seed',
     'given',
-    'strategy',
     'find',
     'example',
     'note',
