@@ -27,18 +27,16 @@ import os
 import inspect
 import warnings
 import threading
+from enum import Enum, unique, IntEnum
 from collections import namedtuple
 
 from hypothesis.errors import InvalidArgument, HypothesisDeprecationWarning
-from hypothesis.configuration import storage_directory, \
-    hypothesis_home_dir, set_hypothesis_home_dir
+from hypothesis.configuration import hypothesis_home_dir
 from hypothesis.utils.conventions import not_set
-from hypothesis.internal.reflection import proxies
 from hypothesis.utils.dynamicvariables import DynamicVariable
 
 __all__ = [
-    'hypothesis_home_dir', 'set_hypothesis_home_dir', 'settings',
-    'storage_directory', 'Settings',
+    'settings',
 ]
 
 
@@ -48,7 +46,7 @@ all_settings = {}
 _db_cache = {}
 
 
-class SettingsProperty(object):
+class settingsProperty(object):
 
     def __init__(self, name):
         self.name = name
@@ -73,38 +71,42 @@ class SettingsProperty(object):
 
     @property
     def __doc__(self):
-        return u'\n'.join((
+        return '\n'.join((
             all_settings[self.name].description,
-            u'default value: %r' % (getattr(settings.default, self.name),)
+            'default value: %r' % (getattr(settings.default, self.name),)
         ))
 
 default_variable = DynamicVariable(None)
 
 
-class SettingsMeta(type):
+class settingsMeta(type):
 
     def __init__(self, *args, **kwargs):
-        super(SettingsMeta, self).__init__(*args, **kwargs)
+        super(settingsMeta, self).__init__(*args, **kwargs)
 
     @property
     def default(self):
+        v = default_variable.value
+        if v is not None:
+            return v
+        if hasattr(settings, '_current_profile'):
+            default_variable.value = settings.load_profile(
+                settings._current_profile
+            )
+            assert default_variable.value is not None
         return default_variable.value
 
     @default.setter
     def default(self, value):
         if default_variable.value is not None:
-            note_deprecation(
-                'Assigning the default settings has been deprecation and will '
-                'be removed in Hypothesis 2.0. Consider using profiles',
-                value
-            )
+            raise AttributeError('Cannot assign settings.default')
         self._assign_default_internal(value)
 
     def _assign_default_internal(self, value):
         default_variable.value = value
 
 
-class settings(SettingsMeta('settings', (object,), {})):
+class settings(settingsMeta('settings', (object,), {})):
 
     """A settings object controls a variety of parameters that are used in
     falsification. These may control both the falsification strategy and the
@@ -128,7 +130,7 @@ class settings(SettingsMeta('settings', (object,), {})):
                 d = d()
             return d
         else:
-            raise AttributeError(u'settings has no attribute %s' % (name,))
+            raise AttributeError('settings has no attribute %s' % (name,))
 
     def __init__(
             self,
@@ -136,19 +138,22 @@ class settings(SettingsMeta('settings', (object,), {})):
             **kwargs
     ):
         self._construction_complete = False
-        self._database = kwargs.pop(u'database', not_set)
+        self._database = kwargs.pop('database', not_set)
         explicit_kwargs = list(kwargs)
         defaults = parent or settings.default
         if defaults is not None:
             for setting in all_settings.values():
                 if kwargs.get(setting.name, not_set) is not_set:
                     kwargs[setting.name] = getattr(defaults, setting.name)
+                elif setting.validator:
+                    kwargs[setting.name] = setting.validator(
+                        kwargs[setting.name])
             if self._database is not_set:
                 self._database = defaults.database
         for name, value in kwargs.items():
             if name not in all_settings:
                 raise InvalidArgument(
-                    u'Invalid argument %s' % (name,))
+                    'Invalid argument %s' % (name,))
             setattr(self, name, value)
         self.storage = threading.local()
         self._construction_complete = True
@@ -171,6 +176,7 @@ class settings(SettingsMeta('settings', (object,), {})):
     @classmethod
     def define_setting(
         cls, name, description, default, options=None, deprecation=None,
+        validator=None,
     ):
         """Add a new setting.
 
@@ -183,65 +189,67 @@ class settings(SettingsMeta('settings', (object,), {})):
 
         """
         if settings.__definitions_are_locked:
-            note_deprecation(
-                'Defining additional settings has been deprecation and will '
-                'be removed in Hypothesis 2.0. Consider managing your '
-                'settings separately.', settings.default
+            from hypothesis.errors import InvalidState
+            raise InvalidState(
+                'settings have been locked and may no longer be defined.'
             )
         if options is not None:
             options = tuple(options)
             if default not in options:
                 raise InvalidArgument(
-                    u'Default value %r is not in options %r' % (
+                    'Default value %r is not in options %r' % (
                         default, options
                     )
                 )
 
         all_settings[name] = Setting(
-            name, description.strip(), default, options, deprecation)
-        setattr(settings, name, SettingsProperty(name))
-        if settings.default:
-            setattr(settings.default, name, default)
-        for profile in settings._profiles.values():
-            setattr(profile, name, default)
+            name, description.strip(), default, options, deprecation, validator
+        )
+        setattr(settings, name, settingsProperty(name))
 
     @classmethod
     def lock_further_definitions(cls):
         settings.__definitions_are_locked = True
 
     def __setattr__(self, name, value):
-        if name == 'database':
-            return object.__setattr__(self, '_database', value)
-        elif name in settings._WHITELISTED_REAL_PROPERTIES:
+        if name in settings._WHITELISTED_REAL_PROPERTIES:
             return object.__setattr__(self, name, value)
-        elif name in all_settings:
-            setting = all_settings[name]
-            if (
-                setting.options is not None and
-                value not in setting.options
-            ):
-                raise InvalidArgument(
-                    u'Invalid %s, %r. Valid options: %r' % (
-                        name, value, setting.options
-                    )
-                )
+        elif name == 'database':
             if self._construction_complete:
-                note_deprecation(
-                    'Mutability of settings is deprecation and will go away '
-                    'in Hypothesis 2.0',
-                    self,
+                raise AttributeError(
+                    'settings objects are immutable and may not be assigned to'
+                    ' after construction.'
                 )
-            return object.__setattr__(self, name, value)
+            else:
+                return object.__setattr__(self, '_database', value)
+        elif name in all_settings:
+            if self._construction_complete:
+                raise AttributeError(
+                    'settings objects are immutable and may not be assigned to'
+                    ' after construction.'
+                )
+            else:
+                setting = all_settings[name]
+                if (
+                    setting.options is not None and
+                    value not in setting.options
+                ):
+                    raise InvalidArgument(
+                        'Invalid %s, %r. Valid options: %r' % (
+                            name, value, setting.options
+                        )
+                    )
+                return object.__setattr__(self, name, value)
         else:
-            raise AttributeError(u'No such setting %s' % (name,))
+            raise AttributeError('No such setting %s' % (name,))
 
     def __repr__(self):
         bits = []
         for name in all_settings:
             value = getattr(self, name)
-            bits.append(u'%s=%r' % (name, value))
+            bits.append('%s=%r' % (name, value))
         bits.sort()
-        return u'settings(%s)' % u', '.join(bits)
+        return 'settings(%s)' % ', '.join(bits)
 
     @property
     def database(self):
@@ -255,16 +263,20 @@ class settings(SettingsMeta('settings', (object,), {})):
         this property is accessed on a particular thread.
 
         """
-        if self._database is not_set and self.database_file is not None:
-            from hypothesis.database import ExampleDatabase
-            from hypothesis.database.backend import SQLiteBackend
-            if self.database_file not in _db_cache:
-                _db_cache[self.database_file] = (
-                    ExampleDatabase(backend=SQLiteBackend(self.database_file)))
-            return _db_cache[self.database_file]
-        if self._database is not_set:
-            self._database = None
-        return self._database
+        try:
+            if self._database is not_set and self.database_file is not None:
+                from hypothesis.database import ExampleDatabase
+                if self.database_file not in _db_cache:
+                    _db_cache[self.database_file] = (
+                        ExampleDatabase(self.database_file))
+                return _db_cache[self.database_file]
+            if self._database is not_set:
+                self._database = None
+            return self._database
+        except AttributeError:
+            import traceback
+            traceback.print_exc()
+            assert False
 
     def __enter__(self):
         default_context_manager = default_variable.with_value(self)
@@ -315,16 +327,18 @@ class settings(SettingsMeta('settings', (object,), {})):
         defined default for that setting
 
         """
+        settings._current_profile = name
         settings._assign_default_internal(settings.get_profile(name))
 
 
 Setting = namedtuple(
-    u'Setting', (
-        u'name', u'description', u'default', u'options', 'deprecation'))
+    'Setting', (
+        'name', 'description', 'default', 'options', 'deprecation', 'validator'
+    ))
 
 
 settings.define_setting(
-    u'min_satisfying_examples',
+    'min_satisfying_examples',
     default=5,
     description="""
 Raise Unsatisfiable for any tests which do not produce at least this many
@@ -334,7 +348,7 @@ search space.
 )
 
 settings.define_setting(
-    u'max_examples',
+    'max_examples',
     default=200,
     description="""
 Once this many satisfying examples have been considered without finding any
@@ -343,7 +357,7 @@ counter-example, falsification will terminate.
 )
 
 settings.define_setting(
-    u'max_iterations',
+    'max_iterations',
     default=1000,
     description="""
 Once this many iterations of the example loop have run, including ones which
@@ -353,7 +367,29 @@ will terminate.
 )
 
 settings.define_setting(
-    u'max_shrinks',
+    'max_mutations',
+    default=10,
+    description="""
+Hypothesis will try this many variations on a single example before moving on
+to an entirely fresh start. If you've got hard to satisfy properties raising
+this might help, but you probably shouldn't touch this dial unless you really
+know what you're doing.
+"""
+)
+
+settings.define_setting(
+    'buffer_size',
+    default=8 * 1024,
+    description="""
+The size of the underlying data used to generate examples. If you need to
+generate really large examples you may want to increase this, but it will make
+your tests slower.
+"""
+)
+
+
+settings.define_setting(
+    'max_shrinks',
     default=500,
     description="""
 Once this many successful shrinks have been performed, Hypothesis will assume
@@ -363,7 +399,7 @@ shrink the example.
 )
 
 settings.define_setting(
-    u'timeout',
+    'timeout',
     default=60,
     description="""
 Once this many seconds have passed, falsify will terminate even
@@ -375,7 +411,7 @@ applied.
 )
 
 settings.define_setting(
-    u'derandomize',
+    'derandomize',
     default=False,
     description="""
 If this is True then hypothesis will run in deterministic mode
@@ -389,8 +425,8 @@ find novel breakages.
 )
 
 settings.define_setting(
-    u'strict',
-    default=os.getenv(u'HYPOTHESIS_STRICT_MODE') == u'true',
+    'strict',
+    default=os.getenv('HYPOTHESIS_STRICT_MODE') == 'true',
     description="""
 If set to True, anything that would cause Hypothesis to issue a warning will
 instead raise an error. Note that new warnings may be added at any time, so
@@ -403,10 +439,10 @@ environment variable to the string 'true'.
 )
 
 settings.define_setting(
-    u'database_file',
+    'database_file',
     default=lambda: (
-        os.getenv(u'HYPOTHESIS_DATABASE_FILE') or
-        os.path.join(hypothesis_home_dir(), u'examples.db')
+        os.getenv('HYPOTHESIS_DATABASE_FILE') or
+        os.path.join(hypothesis_home_dir(), 'examples')
     ),
     description="""
     database: An instance of hypothesis.database.ExampleDatabase that will be
@@ -416,10 +452,28 @@ in which case no storage will be used.
 )
 
 
+@unique
+class Phase(IntEnum):
+    explicit = 0
+    reuse = 1
+    generate = 2
+    shrink = 3
+
+
+@unique
+class HealthCheck(Enum):
+    exception_in_generation = 0
+    data_too_large = 1
+    filter_too_much = 2
+    too_slow = 3
+    random_module = 4
+    return_value = 5
+
+
 class Verbosity(object):
 
     def __repr__(self):
-        return u'Verbosity.%s' % (self.name,)
+        return 'Verbosity.%s' % (self.name,)
 
     def __init__(self, name, level):
         self.name = name
@@ -453,18 +507,18 @@ class Verbosity(object):
         result = getattr(cls, key, None)
         if isinstance(result, Verbosity):
             return result
-        raise InvalidArgument(u'No such verbosity level %r' % (key,))
+        raise InvalidArgument('No such verbosity level %r' % (key,))
 
-Verbosity.quiet = Verbosity(u'quiet', 0)
-Verbosity.normal = Verbosity(u'normal', 1)
-Verbosity.verbose = Verbosity(u'verbose', 2)
-Verbosity.debug = Verbosity(u'debug', 3)
+Verbosity.quiet = Verbosity('quiet', 0)
+Verbosity.normal = Verbosity('normal', 1)
+Verbosity.verbose = Verbosity('verbose', 2)
+Verbosity.debug = Verbosity('debug', 3)
 Verbosity.all = [
     Verbosity.quiet, Verbosity.normal, Verbosity.verbose, Verbosity.debug
 ]
 
 
-ENVIRONMENT_VERBOSITY_OVERRIDE = os.getenv(u'HYPOTHESIS_VERBOSITY_LEVEL')
+ENVIRONMENT_VERBOSITY_OVERRIDE = os.getenv('HYPOTHESIS_VERBOSITY_LEVEL')
 
 if ENVIRONMENT_VERBOSITY_OVERRIDE:
     DEFAULT_VERBOSITY = Verbosity.by_name(ENVIRONMENT_VERBOSITY_OVERRIDE)
@@ -472,14 +526,32 @@ else:
     DEFAULT_VERBOSITY = Verbosity.normal
 
 settings.define_setting(
-    u'verbosity',
+    'verbosity',
     options=Verbosity.all,
     default=DEFAULT_VERBOSITY,
-    description=u'Control the verbosity level of Hypothesis messages',
+    description='Control the verbosity level of Hypothesis messages',
+)
+
+
+def _validate_phases(phases):
+    if phases is None:
+        return tuple(Phase)
+    phases = tuple(phases)
+    for a in phases:
+        if not isinstance(a, Phase):
+            raise InvalidArgument('%r is not a valid phase' % (a,))
+    return phases
+
+
+settings.define_setting(
+    'phases',
+    default=tuple(Phase),
+    description='Control which phases should be run',
+    validator=_validate_phases,
 )
 
 settings.define_setting(
-    name=u'stateful_step_count',
+    name='stateful_step_count',
     default=50,
     description="""
 Number of steps to run a stateful program for before giving up on it breaking.
@@ -487,16 +559,7 @@ Number of steps to run a stateful program for before giving up on it breaking.
 )
 
 settings.define_setting(
-    u'average_list_length',
-    default=25.0,
-    description=u'Average length of lists to use',
-    deprecation=(
-        'average_list_length has been deprecated. Please specify '
-        'average_size explicitly for strategies that need it.')
-)
-
-settings.define_setting(
-    u'perform_health_check',
+    'perform_health_check',
     default=True,
     description=u"""
 If set to True, Hypothesis will run a preliminary health check before
@@ -504,35 +567,11 @@ attempting to actually execute your test.
 """
 )
 
-
-class Settings(settings):
-
-    def __init__(self, *args, **kwargs):
-        super(Settings, self).__init__(*args, **kwargs)
-        note_deprecation(
-            'Use of Settings is deprecation. The new name is settings. The '
-            'behaviour is otherwise unchanged.', self
-        )
-
-settings.Settings = Settings
-
-
-def install_configuration_proxy_method(name):
-    base = globals()[name]
-
-    @staticmethod
-    @proxies(base)
-    def proxy(*args, **kwargs):
-        note_deprecation(
-            '%s has moved from hypothesis.settings to hypothesis.configuration'
-            '.'
-        )
-        return base(*args, **kwargs)
-    setattr(settings, name, proxy)
-
-install_configuration_proxy_method('hypothesis_home_dir')
-install_configuration_proxy_method('set_hypothesis_home_dir')
-install_configuration_proxy_method('storage_directory')
+settings.define_setting(
+    'suppress_health_check',
+    default=[],
+    description="""A list of health checks to disable"""
+)
 
 
 settings.lock_further_definitions()
