@@ -96,6 +96,9 @@ if PY3:
     def escape_unicode_characters(s):
         return codecs.encode(s, 'unicode_escape').decode('ascii')
 
+    def print_unicode(x):
+        print(x)
+
     exec("""
 def quiet_raise(exc):
     raise exc from None
@@ -220,6 +223,11 @@ else:
     def escape_unicode_characters(s):
         return codecs.encode(s, 'string_escape')
 
+    def print_unicode(x):
+        if isinstance(x, unicode):
+            x = x.encode(a_good_encoding())
+        print(x)
+
     def quiet_raise(exc):
         raise exc
 
@@ -419,6 +427,29 @@ class compatbytes(bytearray):
 if PY2:
     hbytes = compatbytes
     reasonable_byte_type = bytearray
+    string_types = (str, unicode)
 else:
     hbytes = bytes
     reasonable_byte_type = bytes
+    string_types = (str,)
+
+
+if PY2:
+    def to_str(s):
+        if isinstance(s, unicode):
+            return s.encode(a_good_encoding())
+        assert isinstance(s, str)
+        return s
+else:
+    def to_str(s):
+        return s
+
+
+def cast_unicode(s, encoding=None):
+    if isinstance(s, bytes):
+        return s.decode(encoding or a_good_encoding(), 'replace')
+    return s
+
+
+def get_stream_enc(stream, default=None):
+    return getattr(stream, 'encoding', None) or default
