@@ -18,29 +18,28 @@ from __future__ import division, print_function, absolute_import
 
 from contextlib import contextmanager
 
-from hypothesis.errors import BadTemplateDraw
 from hypothesis.searchstrategy.wrappers import WrapperStrategy
-from hypothesis.searchstrategy.strategies import OneOfStrategy
+from hypothesis.searchstrategy.strategies import OneOfStrategy, \
+    SearchStrategy
 
 
-class TemplateLimitReached(BaseException):
+class LimitReached(BaseException):
     pass
 
 
-class TemplateLimitedStrategy(WrapperStrategy):
+class LimitedStrategy(WrapperStrategy):
 
     def __init__(self, strategy):
-        super(TemplateLimitedStrategy, self).__init__(strategy)
+        super(LimitedStrategy, self).__init__(strategy)
         self.marker = 0
         self.currently_capped = False
 
-    def draw_template(self, random, parameter_value):
-        if self.currently_capped:
-            if self.marker <= 0:
-                raise TemplateLimitReached()
-            self.marker -= 1
-        return super(TemplateLimitedStrategy, self).draw_template(
-            random, parameter_value)
+    def do_draw(self, data):
+        assert self.currently_capped
+        if self.marker <= 0:
+            raise LimitReached()
+        self.marker -= 1
+        return super(LimitedStrategy, self).do_draw(data)
 
     @contextmanager
     def capped(self, max_templates):
@@ -53,26 +52,27 @@ class TemplateLimitedStrategy(WrapperStrategy):
             self.currently_capped = False
 
 
-class RecursiveStrategy(WrapperStrategy):
+class RecursiveStrategy(SearchStrategy):
 
     def __init__(self, base, extend, max_leaves):
         self.max_leaves = max_leaves
-        self.base = TemplateLimitedStrategy(base)
+        self.base = LimitedStrategy(base)
         self.extend = extend
 
         strategies = [self.base, self.extend(self.base)]
         while 2 ** len(strategies) <= max_leaves:
             strategies.append(
-                extend(OneOfStrategy(tuple(strategies))))
-        super(RecursiveStrategy, self).__init__(
-            OneOfStrategy(tuple(strategies))
-        )
+                extend(OneOfStrategy(tuple(strategies), bias=0.8)))
+        self.strategy = OneOfStrategy(strategies)
 
-    def draw_template(self, random, pv):
-        try:
-            with self.base.capped(self.max_leaves):
-                return super(RecursiveStrategy, self).draw_template(
-                    random, pv
-                )
-        except TemplateLimitReached:
-            raise BadTemplateDraw()
+    def validate(self):
+        self.base.validate()
+        self.extend(self.base).validate()
+
+    def do_draw(self, data):
+        while True:
+            try:
+                with self.base.capped(self.max_leaves):
+                    return data.draw(self.strategy)
+            except LimitReached:
+                pass
